@@ -79,7 +79,7 @@ namespace transport
 				nonZero++;
 				if (nonZero - sharedKey > 32)
 				{
-					LogPrint (eLogWarning, "SSU: first 32 bytes of shared key is all zeros. Ignored");
+					LogPrint (eLogWarning, "SSU: First 32 bytes of shared key is all zeros. Ignored");
 					return;
 				}
 			}
@@ -124,7 +124,7 @@ namespace transport
 						i2p::context.GetRouterInfo ().GetSSUAddress (true);
 					if (!address)
 					{
-						LogPrint (eLogInfo, "SSU is not supported");
+						LogPrint (eLogInfo, "SSU: SSU is not supported");
 						return;
 					}
 					if (Validate (buf, len, address->ssu->key))
@@ -158,7 +158,7 @@ namespace transport
 		auto headerSize = GetSSUHeaderSize (buf);
 		if (headerSize >= len)
 		{
-			LogPrint (eLogError, "SSU header size ", headerSize, " exceeds packet length ", len);
+			LogPrint (eLogError, "SSU: SSU header size ", headerSize, " exceeds packet length ", len);
 			return;
 		}
 		SSUHeader * header = (SSUHeader *)buf;
@@ -177,12 +177,12 @@ namespace transport
 				ProcessSessionConfirmed (buf, len); // buf with header
 			break;
 			case PAYLOAD_TYPE_PEER_TEST:
-				LogPrint (eLogDebug, "SSU: peer test received");
+				LogPrint (eLogDebug, "SSU: Peer test received");
 				ProcessPeerTest (buf + headerSize, len - headerSize, senderEndpoint);
 			break;
 			case PAYLOAD_TYPE_SESSION_DESTROYED:
 			{
-				LogPrint (eLogDebug, "SSU: session destroy received");
+				LogPrint (eLogDebug, "SSU: Session destroy received");
 				m_Server.DeleteSession (shared_from_this ());
 				break;
 			}
@@ -192,11 +192,11 @@ namespace transport
 					m_Server.DeleteSession (shared_from_this ());
 			break;
 			case PAYLOAD_TYPE_RELAY_REQUEST:
-				LogPrint (eLogDebug, "SSU: relay request received");
+				LogPrint (eLogDebug, "SSU: Relay request received");
 				ProcessRelayRequest (buf + headerSize, len - headerSize, senderEndpoint);
 			break;
 			case PAYLOAD_TYPE_RELAY_INTRO:
-				LogPrint (eLogDebug, "SSU: relay intro received");
+				LogPrint (eLogDebug, "SSU: Relay intro received");
 				ProcessRelayIntro (buf + headerSize, len - headerSize);
 			break;
 			default:
@@ -206,7 +206,7 @@ namespace transport
 
 	void SSUSession::ProcessSessionRequest (const uint8_t * buf, size_t len)
 	{
-		LogPrint (eLogDebug, "SSU message: session request");
+		LogPrint (eLogDebug, "SSU message: Session request");
 		bool sendRelayTag = true;
 		auto headerSize = sizeof (SSUHeader);
 		if (((SSUHeader *)buf)->IsExtendedOptions ())
@@ -222,7 +222,7 @@ namespace transport
 		}
 		if (headerSize >= len)
 		{
-			LogPrint (eLogError, "Session request header size ", headerSize, " exceeds packet length ", len);
+			LogPrint (eLogError, "SSU message: Session request header size ", headerSize, " exceeds packet length ", len);
 			return;
 		}
 		if (!m_DHKeysPair)
@@ -230,7 +230,7 @@ namespace transport
 			auto pair = std::make_shared<i2p::crypto::DHKeys> ();
 			pair->GenerateKeys ();
 			m_DHKeysPair = pair;
-		}	
+		}
 		CreateAESandMacKey (buf + headerSize);
 		SendSessionCreated (buf + headerSize, sendRelayTag);
 	}
@@ -249,7 +249,7 @@ namespace transport
 		auto headerSize = GetSSUHeaderSize (buf);
 		if (headerSize >= len)
 		{
-			LogPrint (eLogError, "Session created header size ", headerSize, " exceeds packet length ", len);
+			LogPrint (eLogError, "SSU message: Session created header size ", headerSize, " exceeds packet length ", len);
 			return;
 		}
 		uint8_t * payload = buf + headerSize;
@@ -259,7 +259,7 @@ namespace transport
 		s.Insert (y, 256); // y
 		payload += 256;
 		boost::asio::ip::address ourIP;
-		uint16_t ourPort = 0; 
+		uint16_t ourPort = 0;
 		auto addressAndPortLen = ExtractIPAddressAndPort (payload, len, ourIP, ourPort);
 		if (!addressAndPortLen) return;
 		uint8_t * ourAddressAndPort = payload + 1;
@@ -280,7 +280,7 @@ namespace transport
 			uint32_t signedOnTime = bufbe32toh(payload);
 			if (signedOnTime < ts - SSU_CLOCK_SKEW || signedOnTime > ts + SSU_CLOCK_SKEW)
 			{
-				LogPrint (eLogError, "SSU: clock skew detected ", (int)ts - signedOnTime, ". Check your clock");
+				LogPrint (eLogError, "SSU: Clock skew detected ", (int)ts - signedOnTime, ". Check your clock");
 				i2p::context.SetError (eRouterErrorClockSkew);
 			}
 		}
@@ -297,19 +297,19 @@ namespace transport
 		{
 			LogPrint (eLogInfo, "SSU: Our external address is ", ourIP.to_string (), ":", ourPort);
 			if (!i2p::util::net::IsInReservedRange (ourIP))
-			{	
+			{
 				i2p::context.UpdateAddress (ourIP);
-				SendSessionConfirmed (y, ourAddressAndPort, addressAndPortLen); 
-			}	
+				SendSessionConfirmed (y, ourAddressAndPort, addressAndPortLen);
+			}
 			else
-			{	
-				LogPrint (eLogError, "SSU: Wrong external address ", ourIP.to_string ());
+			{
+				LogPrint (eLogError, "SSU: External address ", ourIP.to_string (), " is in reserved range");
 				Failed ();
-			}	
+			}
 		}
 		else
 		{
-			LogPrint (eLogError, "SSU: message 'created' signature verification failed");
+			LogPrint (eLogError, "SSU: Message 'created' signature verification failed");
 			Failed ();
 		}
 	}
@@ -317,7 +317,7 @@ namespace transport
 	void SSUSession::ProcessSessionConfirmed (const uint8_t * buf, size_t len)
 	{
 		LogPrint (eLogDebug, "SSU: Session confirmed received");
-		m_ConnectTimer.cancel (); 
+		m_ConnectTimer.cancel ();
 		auto headerSize = GetSSUHeaderSize (buf);
 		if (headerSize >= len)
 		{
@@ -331,7 +331,7 @@ namespace transport
 		{
 			LogPrint (eLogError, "SSU: Session confirmed identity size ", identitySize, " exceeds packet length ", len);
 			return;
-		}	
+		}
 		payload += 2; // size of identity fragment
 		auto identity = std::make_shared<i2p::data::IdentityEx> (payload, identitySize);
 		auto existing = i2p::data::netdb.FindRouter (identity->GetIdentHash ()); // check if exists already
@@ -342,7 +342,7 @@ namespace transport
 		uint32_t signedOnTime = bufbe32toh(payload);
 		if (signedOnTime < ts - SSU_CLOCK_SKEW || signedOnTime > ts + SSU_CLOCK_SKEW)
 		{
-			LogPrint (eLogError, "SSU message 'confirmed' time difference ", (int)ts - signedOnTime, " exceeds clock skew");
+			LogPrint (eLogError, "SSU: Message 'confirmed' time difference ", (int)ts - signedOnTime, " exceeds clock skew");
 			Failed ();
 			return;
 		}
@@ -357,7 +357,7 @@ namespace transport
 		{
 			LogPrint (eLogError, "SSU: Session confirmed message is too short ", len);
 			return;
-		}	
+		}
 		// verify signature
 		if (m_SignedData && m_SignedData->Verify (m_RemoteIdentity, payload))
 		{
@@ -366,7 +366,7 @@ namespace transport
 		}
 		else
 		{
-			LogPrint (eLogError, "SSU message 'confirmed' signature verification failed");
+			LogPrint (eLogError, "SSU: Message 'confirmed' signature verification failed");
 			Failed ();
 		}
 	}
@@ -413,7 +413,7 @@ namespace transport
 			i2p::context.GetRouterInfo ().GetSSUAddress (true);
 		if (!address)
 		{
-			LogPrint (eLogInfo, "SSU is not supported");
+			LogPrint (eLogInfo, "SSU: SSU is not supported");
 			return;
 		}
 
@@ -438,7 +438,7 @@ namespace transport
 		else
 			FillHeaderAndEncrypt (PAYLOAD_TYPE_RELAY_REQUEST, buf, 96, introducer.iKey, iv, introducer.iKey);
 		m_Server.Send (buf, 96, m_RemoteEndpoint);
-		LogPrint (eLogDebug, "SSU: relay request sent");
+		LogPrint (eLogDebug, "SSU: Relay request sent");
 	}
 
 	void SSUSession::SendSessionCreated (const uint8_t * x, bool sendRelayTag)
@@ -447,7 +447,7 @@ namespace transport
 			i2p::context.GetRouterInfo ().GetSSUAddress (true); //v4 only
 		if (!address)
 		{
-			LogPrint (eLogInfo, "SSU is not supported");
+			LogPrint (eLogInfo, "SSU: SSU is not supported");
 			return;
 		}
 		SignedData s; // x,y, remote IP, remote port, our IP, our port, relayTag, signed on time
@@ -599,19 +599,19 @@ namespace transport
 		uint8_t * payload = buf + sizeof (SSUHeader);
 		// Charlie
 		if (isV4)
-		{	
+		{
 			*payload = 4;
 			payload++; // size
 			memcpy (payload, to.address ().to_v4 ().to_bytes ().data (), 4); // Charlie's IP V4
 			payload += 4; // address
-		}	
+		}
 		else
 		{
 			*payload = 16;
 			payload++; // size
-			memcpy (payload, to.address ().to_v6 ().to_bytes ().data (), 16); // Alice's IP V6
+			memcpy (payload, to.address ().to_v6 ().to_bytes ().data (), 16); // Charlie's IP V6
 			payload += 16; // address
-		}	
+		}
 		htobe16buf (payload, to.port ()); // Charlie's port
 		payload += 2; // port
 		// Alice
@@ -647,7 +647,7 @@ namespace transport
 			FillHeaderAndEncrypt (PAYLOAD_TYPE_RELAY_RESPONSE, buf, isV4 ? 64 : 80, introKey, iv, introKey);
 			m_Server.Send (buf, isV4 ? 64 : 80, from);
 		}
-		LogPrint (eLogDebug, "SSU: relay response sent");
+		LogPrint (eLogDebug, "SSU: Relay response sent");
 	}
 
 	void SSUSession::SendRelayIntro (std::shared_ptr<SSUSession> session, const boost::asio::ip::udp::endpoint& from)
@@ -683,7 +683,7 @@ namespace transport
 		RAND_bytes (iv, 16); // random iv
 		FillHeaderAndEncrypt (PAYLOAD_TYPE_RELAY_INTRO, buf, isV4 ? 48 : 64, session->m_SessionKey, iv, session->m_MacKey);
 		m_Server.Send (buf, isV4 ? 48 : 64, session->m_RemoteEndpoint);
-		LogPrint (eLogDebug, "SSU: relay intro sent");
+		LogPrint (eLogDebug, "SSU: Relay intro sent");
 	}
 
 	void SSUSession::ProcessRelayResponse (const uint8_t * buf, size_t len)
@@ -703,17 +703,17 @@ namespace transport
 		if (!i2p::util::net::IsInReservedRange (ourIP))
 			i2p::context.UpdateAddress (ourIP);
 		else
-			LogPrint (eLogWarning, "SSU: Wrong external address ", ourIP.to_string ());
+			LogPrint (eLogError, "SSU: External address ", ourIP.to_string (), " is in reserved range");
 		if (ourIP.is_v4 ())
-		{	
+		{
 			if (ourPort != m_Server.GetPort ())
-			{	
+			{
 				if (i2p::context.GetStatus () == eRouterStatusTesting)
 					i2p::context.SetError (eRouterErrorSymmetricNAT);
 			}
 			else if (i2p::context.GetStatus () == eRouterStatusError && i2p::context.GetError () == eRouterErrorSymmetricNAT)
 				i2p::context.SetStatus (eRouterStatusTesting);
-		}	
+		}
 		uint32_t nonce = bufbe32toh (buf);
 		buf += 4; // nonce
 		auto it = m_RelayRequests.find (nonce);
@@ -727,10 +727,10 @@ namespace transport
 				// now we do
 				LogPrint (eLogInfo, "SSU: RelayReponse connecting to endpoint ", remoteEndpoint);
 				if ((remoteIP.is_v4 () && i2p::context.GetStatus () == eRouterStatusFirewalled) ||
-					(remoteIP.is_v6 () && i2p::context.GetStatusV6 () == eRouterStatusFirewalled)) 
+					(remoteIP.is_v6 () && i2p::context.GetStatusV6 () == eRouterStatusFirewalled))
 					m_Server.Send (buf, 0, remoteEndpoint); // send HolePunch
 				// we assume that HolePunch has been sent by this time and our SessionRequest will go through
-				m_Server.CreateDirectSession (it->second, remoteEndpoint, false);
+				m_Server.CreateDirectSession (it->second.first, remoteEndpoint, false);
 			}
 			// delete request
 			m_RelayRequests.erase (it);
@@ -803,7 +803,7 @@ namespace transport
 		htobe16buf (out + len + 16, (netid == I2PD_NET_ID) ? encryptedLen : encryptedLen ^ ((netid - 2) << 8));
 		i2p::crypto::HMACMD5Digest (encrypted, encryptedLen + 18, m_MacKey, header->mac);
 	}
-		
+
 	void SSUSession::Decrypt (uint8_t * buf, size_t len, const i2p::crypto::AESKey& aesKey)
 	{
 		if (len < sizeof (SSUHeader))
@@ -872,7 +872,7 @@ namespace transport
 		if (!IsOutgoing ()) // incoming session
 			ScheduleConnectTimer ();
 		else
-			LogPrint (eLogError, "SSU: wait for connect for outgoing session");
+			LogPrint (eLogError, "SSU: Wait for connect for outgoing session");
 	}
 
 	void SSUSession::ScheduleConnectTimer ()
@@ -888,7 +888,7 @@ namespace transport
 		if (!ecode)
 		{
 			// timeout expired
-			LogPrint (eLogWarning, "SSU: session with ", m_RemoteEndpoint, " was not established after ", SSU_CONNECT_TIMEOUT, " seconds");
+			LogPrint (eLogWarning, "SSU: Session with ", m_RemoteEndpoint, " was not established after ", SSU_CONNECT_TIMEOUT, " seconds");
 			Failed ();
 		}
 	}
@@ -905,7 +905,8 @@ namespace transport
 		}
 		uint32_t nonce;
 		RAND_bytes ((uint8_t *)&nonce, 4);
-		m_RelayRequests[nonce] = to;
+		auto ts = i2p::util::GetSecondsSinceEpoch ();
+		m_RelayRequests.emplace (nonce, std::make_pair (to, ts));
 		SendRelayRequest (introducer, nonce);
 	}
 
@@ -1004,10 +1005,22 @@ namespace transport
 		}
 	}
 
+	void SSUSession::CleanUp (uint64_t ts)
+	{
+		m_Data.CleanUp (ts);
+		for (auto it = m_RelayRequests.begin (); it != m_RelayRequests.end ();)
+		{
+			if (ts > it->second.second + SSU_CONNECT_TIMEOUT)
+				it =  m_RelayRequests.erase (it);
+			else
+				++it;
+		}
+	}
+
 	void SSUSession::ProcessPeerTest (const uint8_t * buf, size_t len, const boost::asio::ip::udp::endpoint& senderEndpoint)
 	{
 		uint32_t nonce = bufbe32toh (buf); // 4 bytes
-		boost::asio::ip::address addr; // Alice's addresss
+		boost::asio::ip::address addr; // Alice's address
 		uint16_t port = 0; // and port
 		auto size = ExtractIPAddressAndPort (buf + 4, len - 4, addr, port);
 		if (port && (size != 7) && (size != 19))
@@ -1023,29 +1036,29 @@ namespace transport
 			{
 				if (m_Server.GetPeerTestSession (nonce) == shared_from_this ()) // Alice-Bob
 				{
-					LogPrint (eLogDebug, "SSU: peer test from Bob. We are Alice");
+					LogPrint (eLogDebug, "SSU: Peer test from Bob. We are Alice");
 					if (IsV6 ())
 					{
-						if (i2p::context.GetStatusV6 () == eRouterStatusTesting) 
-						{	
+						if (i2p::context.GetStatusV6 () == eRouterStatusTesting)
+						{
 							i2p::context.SetStatusV6 (eRouterStatusFirewalled);
 							m_Server.RescheduleIntroducersUpdateTimerV6 ();
-						}	
-					}	
+						}
+					}
 					else if (i2p::context.GetStatus () == eRouterStatusTesting) // still not OK
-					{	
+					{
 						i2p::context.SetStatus (eRouterStatusFirewalled);
 						m_Server.RescheduleIntroducersUpdateTimer ();
-					}	
+					}
 				}
 				else
 				{
-					LogPrint (eLogDebug, "SSU: first peer test from Charlie. We are Alice");
+					LogPrint (eLogDebug, "SSU: First peer test from Charlie. We are Alice");
 					if (m_State == eSessionStateEstablished)
-						LogPrint (eLogWarning, "SSU: first peer test from Charlie through established session. We are Alice");
+						LogPrint (eLogWarning, "SSU: First peer test from Charlie through established session. We are Alice");
 					if (IsV6 ())
 						i2p::context.SetStatusV6 (eRouterStatusOK);
-					else	
+					else
 						i2p::context.SetStatus (eRouterStatusOK);
 					m_Server.UpdatePeerTest (nonce, ePeerTestParticipantAlice2);
 					SendPeerTest (nonce, senderEndpoint.address (), senderEndpoint.port (), introKey, true, false); // to Charlie
@@ -1055,14 +1068,14 @@ namespace transport
 			case ePeerTestParticipantAlice2:
 			{
 				if (m_Server.GetPeerTestSession (nonce) == shared_from_this ()) // Alice-Bob
-					LogPrint (eLogDebug, "SSU: peer test from Bob. We are Alice");
+					LogPrint (eLogDebug, "SSU: Peer test from Bob. We are Alice");
 				else
 				{
 					// peer test successive
-					LogPrint (eLogDebug, "SSU: second peer test from Charlie. We are Alice");
+					LogPrint (eLogDebug, "SSU: Second peer test from Charlie. We are Alice");
 					if (IsV6 ())
-						i2p::context.SetStatusV6 (eRouterStatusOK);		
-					else		
+						i2p::context.SetStatusV6 (eRouterStatusOK);
+					else
 						i2p::context.SetStatus (eRouterStatusOK);
 					m_Server.RemovePeerTest (nonce);
 				}
@@ -1070,19 +1083,19 @@ namespace transport
 			}
 			case ePeerTestParticipantBob:
 			{
-				LogPrint (eLogDebug, "SSU: peer test from Charlie. We are Bob");
+				LogPrint (eLogDebug, "SSU: Peer test from Charlie. We are Bob");
 				auto session = m_Server.GetPeerTestSession (nonce); // session with Alice from PeerTest
 				if (session && session->m_State == eSessionStateEstablished)
 				{
 					const auto& ep = session->GetRemoteEndpoint (); // Alice's endpoint as known to Bob
 					session->SendPeerTest (nonce, ep.address (), ep.port (), introKey, false, true); // send back to Alice
-				}	
+				}
 				m_Server.RemovePeerTest (nonce); // nonce has been used
 				break;
 			}
 			case ePeerTestParticipantCharlie:
 			{
-				LogPrint (eLogDebug, "SSU: peer test from Alice. We are Charlie");
+				LogPrint (eLogDebug, "SSU: Peer test from Alice. We are Charlie");
 				SendPeerTest (nonce, senderEndpoint.address (), senderEndpoint.port (), introKey); // to Alice with her actual address
 				m_Server.RemovePeerTest (nonce); // nonce has been used
 				break;
@@ -1095,17 +1108,17 @@ namespace transport
 					// new test
 					if (port)
 					{
-						LogPrint (eLogDebug, "SSU: peer test from Bob. We are Charlie");
+						LogPrint (eLogDebug, "SSU: Peer test from Bob. We are Charlie");
 						Send (PAYLOAD_TYPE_PEER_TEST, buf, len); // back to Bob
 						if (!addr.is_unspecified () && !i2p::util::net::IsInReservedRange(addr))
-						{	
+						{
 							m_Server.NewPeerTest (nonce, ePeerTestParticipantCharlie);
 							SendPeerTest (nonce, addr, port, introKey); // to Alice with her address received from Bob
-						}	
+						}
 					}
 					else
 					{
-						LogPrint (eLogDebug, "SSU: peer test from Alice. We are Bob");
+					LogPrint (eLogDebug, "SSU: Peer test from Alice. We are Bob");
 						auto session = senderEndpoint.address ().is_v4 () ? m_Server.GetRandomEstablishedV4Session (shared_from_this ()) : m_Server.GetRandomEstablishedV6Session (shared_from_this ()); // Charlie
 						if (session)
 						{
@@ -1115,7 +1128,7 @@ namespace transport
 					}
 				}
 				else
-					LogPrint (eLogError, "SSU: unexpected peer test");
+					LogPrint (eLogError, "SSU: Unexpected peer test");
 			}
 		}
 	}
@@ -1163,7 +1176,7 @@ namespace transport
 			if (addr)
 				memcpy (payload, addr->ssu->key, 32); // intro key
 			else
-				LogPrint (eLogInfo, "SSU is not supported. Can't send peer test");
+				LogPrint (eLogInfo, "SSU: SSU is not supported. Can't send peer test");
 		}
 		else
 			memcpy (payload, introKey, 32); // intro key
@@ -1188,11 +1201,11 @@ namespace transport
 	void SSUSession::SendPeerTest ()
 	{
 		// we are Alice
-		LogPrint (eLogDebug, "SSU: sending peer test");
+		LogPrint (eLogDebug, "SSU: Sending peer test");
 		auto address = IsV6 () ? i2p::context.GetRouterInfo ().GetSSUV6Address () : i2p::context.GetRouterInfo ().GetSSUAddress (true);
 		if (!address)
 		{
-			LogPrint (eLogInfo, "SSU is not supported. Can't send peer test");
+			LogPrint (eLogInfo, "SSU: SSU is not supported. Can't send peer test");
 			return;
 		}
 		uint32_t nonce;
@@ -1233,9 +1246,9 @@ namespace transport
 			}
 			catch (std::exception& ex)
 			{
-				LogPrint (eLogWarning, "SSU: exception while sending session destoroyed: ", ex.what ());
+				LogPrint (eLogWarning, "SSU: Exception while sending session destoroyed: ", ex.what ());
 			}
-			LogPrint (eLogDebug, "SSU: session destroyed sent");
+			LogPrint (eLogDebug, "SSU: Session destroyed sent");
 		}
 	}
 
@@ -1247,7 +1260,7 @@ namespace transport
 		if (paddingSize > 0) msgSize += (16 - paddingSize);
 		if (msgSize > SSU_MTU_V4)
 		{
-			LogPrint (eLogWarning, "SSU: payload size ", msgSize, " exceeds MTU");
+			LogPrint (eLogWarning, "SSU: Payload size ", msgSize, " exceeds MTU");
 			return;
 		}
 		memcpy (buf + sizeof (SSUHeader), payload, len);
@@ -1268,12 +1281,12 @@ namespace transport
 		if (!len) return 0;
 		uint8_t size = *buf;
 		size_t s = 1 + size + 2; // size + address + port
-		if (len < s) 
+		if (len < s)
 		{
 			LogPrint (eLogWarning, "SSU: Address is too short ", len);
 			port = 0;
 			return len;
-		}	
+		}
 		buf++; // size
 		if (size == 4)
 		{
@@ -1286,12 +1299,12 @@ namespace transport
 			boost::asio::ip::address_v6::bytes_type bytes;
 			memcpy (bytes.data (), buf, 16);
 			ip = boost::asio::ip::address_v6 (bytes);
-		}	
+		}
 		else
-			LogPrint (eLogWarning, "SSU: Address size ", size, " is not supported");
+			LogPrint (eLogWarning, "SSU: Address size ", int(size), " is not supported");
 		buf += size;
 		port = bufbe16toh (buf);
 		return s;
-	}	
+	}
 }
 }
