@@ -27,28 +27,28 @@ namespace tunnel
 	void Path::Add (std::shared_ptr<const i2p::data::RouterInfo> r)
 	{
 		if (r)
-		{	
+		{
 			peers.push_back (r->GetRouterIdentity ());
-			if (r->GetVersion () < i2p::data::NETDB_MIN_SHORT_TUNNEL_BUILD_VERSION || 
+			if (r->GetVersion () < i2p::data::NETDB_MIN_SHORT_TUNNEL_BUILD_VERSION ||
 			    r->GetRouterIdentity ()->GetCryptoKeyType () != i2p::data::CRYPTO_KEY_TYPE_ECIES_X25519_AEAD)
 				isShort = false;
-		}	
-	}	
-	
+		}
+	}
+
 	void Path::Reverse ()
 	{
 		std::reverse (peers.begin (), peers.end ());
-	}	
-	
+	}
+
 	TunnelPool::TunnelPool (int numInboundHops, int numOutboundHops, int numInboundTunnels, int numOutboundTunnels):
 		m_NumInboundHops (numInboundHops), m_NumOutboundHops (numOutboundHops),
 		m_NumInboundTunnels (numInboundTunnels), m_NumOutboundTunnels (numOutboundTunnels),
 		m_IsActive (true), m_CustomPeerSelector(nullptr)
 	{
-		if (m_NumInboundTunnels > TUNNEL_POOL_MAX_INBOUND_TUNNELS_QUANTITY) 
+		if (m_NumInboundTunnels > TUNNEL_POOL_MAX_INBOUND_TUNNELS_QUANTITY)
 			m_NumInboundTunnels = TUNNEL_POOL_MAX_INBOUND_TUNNELS_QUANTITY;
-		if (m_NumOutboundTunnels > TUNNEL_POOL_MAX_OUTBOUND_TUNNELS_QUANTITY) 
-			m_NumOutboundTunnels = TUNNEL_POOL_MAX_OUTBOUND_TUNNELS_QUANTITY;	
+		if (m_NumOutboundTunnels > TUNNEL_POOL_MAX_OUTBOUND_TUNNELS_QUANTITY)
+			m_NumOutboundTunnels = TUNNEL_POOL_MAX_OUTBOUND_TUNNELS_QUANTITY;
 		m_NextManageTime = i2p::util::GetSecondsSinceEpoch () + rand () % TUNNEL_POOL_MANAGE_INTERVAL;
 	}
 
@@ -113,6 +113,17 @@ namespace tunnel
 		if (!m_IsActive) return;
 		{
 			std::unique_lock<std::mutex> l(m_InboundTunnelsMutex);
+			if (createdTunnel->IsRecreated ())
+			{
+				// find and mark old tunnel as expired
+				createdTunnel->SetRecreated (false);
+				for (auto& it: m_InboundTunnels)
+					if (it->IsRecreated () && it->GetNextIdentHash () == createdTunnel->GetNextIdentHash ())
+					{
+						it->SetState (eTunnelStateExpiring);
+						break;
+					}
+			}
 			m_InboundTunnels.insert (createdTunnel);
 		}
 		if (m_LocalDestination)
@@ -168,10 +179,10 @@ namespace tunnel
 				if (it->IsSlow () && !slowTunnel)
 					slowTunnel = it;
 				else
-				{	
+				{
 					v.push_back (it);
 					i++;
-				}	
+				}
 			}
 		}
 		if (slowTunnel && (int)v.size () < (num/2+1))
@@ -179,20 +190,23 @@ namespace tunnel
 		return v;
 	}
 
-	std::shared_ptr<OutboundTunnel> TunnelPool::GetNextOutboundTunnel (std::shared_ptr<OutboundTunnel> excluded) const
+	std::shared_ptr<OutboundTunnel> TunnelPool::GetNextOutboundTunnel (std::shared_ptr<OutboundTunnel> excluded,
+		i2p::data::RouterInfo::CompatibleTransports compatible) const
 	{
 		std::unique_lock<std::mutex> l(m_OutboundTunnelsMutex);
-		return GetNextTunnel (m_OutboundTunnels, excluded);
+		return GetNextTunnel (m_OutboundTunnels, excluded, compatible);
 	}
 
-	std::shared_ptr<InboundTunnel> TunnelPool::GetNextInboundTunnel (std::shared_ptr<InboundTunnel> excluded) const
+	std::shared_ptr<InboundTunnel> TunnelPool::GetNextInboundTunnel (std::shared_ptr<InboundTunnel> excluded,
+		i2p::data::RouterInfo::CompatibleTransports compatible) const
 	{
 		std::unique_lock<std::mutex> l(m_InboundTunnelsMutex);
-		return GetNextTunnel (m_InboundTunnels, excluded);
+		return GetNextTunnel (m_InboundTunnels, excluded, compatible);
 	}
 
 	template<class TTunnels>
-	typename TTunnels::value_type TunnelPool::GetNextTunnel (TTunnels& tunnels, typename TTunnels::value_type excluded) const
+	typename TTunnels::value_type TunnelPool::GetNextTunnel (TTunnels& tunnels,
+		typename TTunnels::value_type excluded, i2p::data::RouterInfo::CompatibleTransports compatible) const
 	{
 		if (tunnels.empty ()) return nullptr;
 		uint32_t ind = rand () % (tunnels.size ()/2 + 1), i = 0;
@@ -200,10 +214,10 @@ namespace tunnel
 		typename TTunnels::value_type tunnel = nullptr;
 		for (const auto& it: tunnels)
 		{
-			if (it->IsEstablished () && it != excluded)
+			if (it->IsEstablished () && it != excluded && (compatible & it->GetFarEndTransports ()))
 			{
-				if (it->IsSlow () || (HasLatencyRequirement() && it->LatencyIsKnown() && 
-				    !it->LatencyFitsRange(m_MinLatency, m_MaxLatency))) 
+				if (it->IsSlow () || (HasLatencyRequirement() && it->LatencyIsKnown() &&
+				    !it->LatencyFitsRange(m_MinLatency, m_MaxLatency)))
 				{
 					i++; skipped = true;
 					continue;
@@ -213,7 +227,7 @@ namespace tunnel
 			}
 			if (i > ind && tunnel) break;
 		}
-		if (!tunnel && skipped) 
+		if (!tunnel && skipped)
 		{
 			ind = rand () % (tunnels.size ()/2 + 1), i = 0;
 			for (const auto& it: tunnels)
@@ -270,12 +284,12 @@ namespace tunnel
 		if (!num && !m_OutboundTunnels.empty () && m_NumOutboundHops > 0)
 		{
 			for (auto it: m_OutboundTunnels)
-			{	
+			{
 				CreatePairedInboundTunnel (it);
 				num++;
 				if (num >= m_NumInboundTunnels) break;
-			}	
-		}	
+			}
+		}
 		for (int i = num; i < m_NumInboundTunnels; i++)
 			CreateInboundTunnel ();
 
@@ -293,7 +307,7 @@ namespace tunnel
 
 		for (auto& it: tests)
 		{
-			LogPrint (eLogWarning, "Tunnels: test of tunnel ", it.first, " failed");
+			LogPrint (eLogWarning, "Tunnels: Test of tunnel ", it.first, " failed");
 			// if test failed again with another tunnel we consider it failed
 			if (it.second.first)
 			{
@@ -357,19 +371,19 @@ namespace tunnel
 	void TunnelPool::ManageTunnels (uint64_t ts)
 	{
 		if (ts > m_NextManageTime)
-		{	
+		{
 			CreateTunnels ();
 			TestTunnels ();
-			m_NextManageTime = ts + TUNNEL_POOL_MANAGE_INTERVAL + (rand () % TUNNEL_POOL_MANAGE_INTERVAL)/2;  
-		}	
-	}	
-		
+			m_NextManageTime = ts + TUNNEL_POOL_MANAGE_INTERVAL + (rand () % TUNNEL_POOL_MANAGE_INTERVAL)/2;
+		}
+	}
+
 	void TunnelPool::ProcessGarlicMessage (std::shared_ptr<I2NPMessage> msg)
 	{
 		if (m_LocalDestination)
 			m_LocalDestination->ProcessGarlicMessage (msg);
 		else
-			LogPrint (eLogWarning, "Tunnels: local destination doesn't exist, dropped");
+			LogPrint (eLogWarning, "Tunnels: Local destination doesn't exist, dropped");
 	}
 
 	void TunnelPool::ProcessDeliveryStatus (std::shared_ptr<I2NPMessage> msg)
@@ -394,23 +408,23 @@ namespace tunnel
 		if (found)
 		{
 			uint64_t dlt = i2p::util::GetMillisecondsSinceEpoch () - timestamp;
-			LogPrint (eLogDebug, "Tunnels: test of ", msgID, " successful. ", dlt, " milliseconds");
+			LogPrint (eLogDebug, "Tunnels: Test of ", msgID, " successful. ", dlt, " milliseconds");
 			uint64_t latency = dlt / 2;
 			// restore from test failed state if any
 			if (test.first)
-			{	
+			{
 				if (test.first->GetState () == eTunnelStateTestFailed)
 					test.first->SetState (eTunnelStateEstablished);
 				// update latency
 				test.first->AddLatencySample(latency);
-			}	
+			}
 			if (test.second)
-			{	
+			{
 				if (test.second->GetState () == eTunnelStateTestFailed)
 					test.second->SetState (eTunnelStateEstablished);
 				// update latency
 				test.second->AddLatencySample(latency);
-			}	
+			}
 		}
 		else
 		{
@@ -424,8 +438,8 @@ namespace tunnel
 	bool TunnelPool::IsExploratory () const
 	{
 		return i2p::tunnel::tunnels.GetExploratoryPool () == shared_from_this ();
-	}	
-		
+	}
+
 	std::shared_ptr<const i2p::data::RouterInfo> TunnelPool::SelectNextHop (std::shared_ptr<const i2p::data::RouterInfo> prevHop, bool reverse) const
 	{
 		auto hop = IsExploratory () ? i2p::data::netdb.GetRandomRouter (prevHop, reverse):
@@ -453,7 +467,7 @@ namespace tunnel
 			(inbound && i2p::transport::transports.GetNumPeers () > 25))
 		{
 			auto r = i2p::transport::transports.GetRandomPeer ();
-			if (r && !r->GetProfile ()->IsBad () && 
+			if (r && r->IsECIES () && !r->GetProfile ()->IsBad () &&
 				(numHops > 1 || (r->IsV4 () && (!inbound || r->IsReachable ())))) // first inbound must be reachable
 			{
 				prevHop = r;
@@ -466,10 +480,11 @@ namespace tunnel
 		{
 			auto hop = nextHop (prevHop, inbound);
 			if (!hop && !i) // if no suitable peer found for first hop, try already connected
-			{	
+			{
 				LogPrint (eLogInfo, "Tunnels: Can't select first hop for a tunnel. Trying already connected");
 				hop = i2p::transport::transports.GetRandomPeer ();
-			}	
+				if (hop && !hop->IsECIES ()) hop = nullptr;
+			}
 			if (!hop)
 			{
 				LogPrint (eLogError, "Tunnels: Can't select next hop for ", prevHop->GetIdentHashBase64 ());
@@ -480,10 +495,11 @@ namespace tunnel
 			{
 				auto hop1 = nextHop (prevHop, true);
 				if (hop1) hop = hop1;
-			}	
+			}
 			prevHop = hop;
 			path.Add (hop);
 		}
+		path.farEndTransports = prevHop->GetCompatibleTransports (inbound); // last hop
 		return true;
 	}
 
@@ -507,13 +523,25 @@ namespace tunnel
 	{
 		int numHops = isInbound ? m_NumInboundHops : m_NumOutboundHops;
 		if (numHops > (int)m_ExplicitPeers->size ()) numHops = m_ExplicitPeers->size ();
-		if (!numHops) return false; 
+		if (!numHops) return false;
 		for (int i = 0; i < numHops; i++)
 		{
 			auto& ident = (*m_ExplicitPeers)[i];
 			auto r = i2p::data::netdb.FindRouter (ident);
 			if (r)
-				path.Add (r);
+			{
+				if (r->IsECIES ())
+				{
+					path.Add (r);
+					if (i == numHops - 1)
+						path.farEndTransports = r->GetCompatibleTransports (isInbound);
+				}
+				else
+				{
+					LogPrint (eLogError, "Tunnels: ElGamal router ", ident.ToBase64 (), " is not supported");
+					return false;
+				}
+			}
 			else
 			{
 				LogPrint (eLogInfo, "Tunnels: Can't find router for ", ident.ToBase64 ());
@@ -526,18 +554,18 @@ namespace tunnel
 
 	void TunnelPool::CreateInboundTunnel ()
 	{
-		auto outboundTunnel = GetNextOutboundTunnel ();
-		if (!outboundTunnel)
-			outboundTunnel = tunnels.GetNextOutboundTunnel ();
 		LogPrint (eLogDebug, "Tunnels: Creating destination inbound tunnel...");
 		Path path;
 		if (SelectPeers (path, true))
 		{
+			auto outboundTunnel = GetNextOutboundTunnel (nullptr, path.farEndTransports);
+			if (!outboundTunnel)
+				outboundTunnel = tunnels.GetNextOutboundTunnel ();
 			std::shared_ptr<TunnelConfig> config;
 			if (m_NumInboundHops > 0)
 			{
 				path.Reverse ();
-				config = std::make_shared<TunnelConfig> (path.peers, path.isShort);
+				config = std::make_shared<TunnelConfig> (path.peers, path.isShort, path.farEndTransports);
 			}
 			auto tunnel = tunnels.CreateInboundTunnel (config, shared_from_this (), outboundTunnel);
 			if (tunnel->IsEstablished ()) // zero hops
@@ -554,55 +582,60 @@ namespace tunnel
 			CreateInboundTunnel ();
 			return;
 		}
-		auto outboundTunnel = GetNextOutboundTunnel ();
+		auto outboundTunnel = GetNextOutboundTunnel (nullptr, tunnel->GetFarEndTransports ());
 		if (!outboundTunnel)
 			outboundTunnel = tunnels.GetNextOutboundTunnel ();
 		LogPrint (eLogDebug, "Tunnels: Re-creating destination inbound tunnel...");
 		std::shared_ptr<TunnelConfig> config;
 		if (m_NumInboundHops > 0 && tunnel->GetPeers().size())
-		{
-			config = std::make_shared<TunnelConfig>(tunnel->GetPeers ());
-		}
+			config = std::make_shared<TunnelConfig>(tunnel->GetPeers (), tunnel->IsShortBuildMessage (), tunnel->GetFarEndTransports ());
 		if (!m_NumInboundHops || config)
 		{
 			auto newTunnel = tunnels.CreateInboundTunnel (config, shared_from_this(), outboundTunnel);
 			if (newTunnel->IsEstablished ()) // zero hops
 				TunnelCreated (newTunnel);
+			else
+				newTunnel->SetRecreated (true);
 		}
 	}
 
 	void TunnelPool::CreateOutboundTunnel ()
 	{
-		auto inboundTunnel = GetNextInboundTunnel ();
-		if (!inboundTunnel)
-			inboundTunnel = tunnels.GetNextInboundTunnel ();
-		if (inboundTunnel)
+		LogPrint (eLogDebug, "Tunnels: Creating destination outbound tunnel...");
+		Path path;
+		if (SelectPeers (path, false))
 		{
-			LogPrint (eLogDebug, "Tunnels: Creating destination outbound tunnel...");
-			Path path;
-			if (SelectPeers (path, false))
+			auto inboundTunnel = GetNextInboundTunnel (nullptr, path.farEndTransports);
+			if (!inboundTunnel)
+				inboundTunnel = tunnels.GetNextInboundTunnel ();
+			if (!inboundTunnel)
 			{
-				std::shared_ptr<TunnelConfig> config;
-				if (m_NumOutboundHops > 0)
-					config = std::make_shared<TunnelConfig>(path.peers, inboundTunnel->GetNextTunnelID (), inboundTunnel->GetNextIdentHash (), path.isShort);
+				LogPrint (eLogError, "Tunnels: Can't create outbound tunnel, no inbound tunnels found");
+				return;
+			}
 
-				std::shared_ptr<OutboundTunnel> tunnel;
-				if (path.isShort)
-				{
-					// TODO: implement it better
-					tunnel = tunnels.CreateOutboundTunnel (config, inboundTunnel->GetTunnelPool ());
-					tunnel->SetTunnelPool (shared_from_this ());
-				}	
-				else	
-					tunnel = tunnels.CreateOutboundTunnel (config, shared_from_this ());
-				if (tunnel && tunnel->IsEstablished ()) // zero hops
-					TunnelCreated (tunnel);
+			if (m_LocalDestination && !m_LocalDestination->SupportsEncryptionType (i2p::data::CRYPTO_KEY_TYPE_ECIES_X25519_AEAD))
+				path.isShort = false; // because can't handle ECIES encrypted reply
+
+			std::shared_ptr<TunnelConfig> config;
+			if (m_NumOutboundHops > 0)
+				config = std::make_shared<TunnelConfig>(path.peers, inboundTunnel->GetNextTunnelID (),
+					inboundTunnel->GetNextIdentHash (), path.isShort, path.farEndTransports);
+
+			std::shared_ptr<OutboundTunnel> tunnel;
+			if (path.isShort)
+			{
+				// TODO: implement it better
+				tunnel = tunnels.CreateOutboundTunnel (config, inboundTunnel->GetTunnelPool ());
+				tunnel->SetTunnelPool (shared_from_this ());
 			}
 			else
-				LogPrint (eLogError, "Tunnels: Can't create outbound tunnel, no peers available");
+				tunnel = tunnels.CreateOutboundTunnel (config, shared_from_this ());
+			if (tunnel && tunnel->IsEstablished ()) // zero hops
+				TunnelCreated (tunnel);
 		}
 		else
-			LogPrint (eLogError, "Tunnels: Can't create outbound tunnel, no inbound tunnels found");
+			LogPrint (eLogError, "Tunnels: Can't create outbound tunnel, no peers available");
 	}
 
 	void TunnelPool::RecreateOutboundTunnel (std::shared_ptr<OutboundTunnel> tunnel)
@@ -612,7 +645,7 @@ namespace tunnel
 			CreateOutboundTunnel ();
 			return;
 		}
-		auto inboundTunnel = GetNextInboundTunnel ();
+		auto inboundTunnel = GetNextInboundTunnel (nullptr, tunnel->GetFarEndTransports ());
 		if (!inboundTunnel)
 			inboundTunnel = tunnels.GetNextInboundTunnel ();
 		if (inboundTunnel)
@@ -621,7 +654,8 @@ namespace tunnel
 			std::shared_ptr<TunnelConfig> config;
 			if (m_NumOutboundHops > 0 && tunnel->GetPeers().size())
 			{
-				config = std::make_shared<TunnelConfig>(tunnel->GetPeers (), inboundTunnel->GetNextTunnelID (), inboundTunnel->GetNextIdentHash ());
+				config = std::make_shared<TunnelConfig>(tunnel->GetPeers (), inboundTunnel->GetNextTunnelID (),
+					inboundTunnel->GetNextIdentHash (), inboundTunnel->IsShortBuildMessage (), tunnel->GetFarEndTransports ());
 			}
 			if (!m_NumOutboundHops || config)
 			{
@@ -638,7 +672,8 @@ namespace tunnel
 	{
 		LogPrint (eLogDebug, "Tunnels: Creating paired inbound tunnel...");
 		auto tunnel = tunnels.CreateInboundTunnel (
-			m_NumOutboundHops > 0 ? std::make_shared<TunnelConfig>(outboundTunnel->GetInvertedPeers ()) : nullptr, 
+			m_NumOutboundHops > 0 ? std::make_shared<TunnelConfig>(outboundTunnel->GetInvertedPeers (),
+				outboundTunnel->IsShortBuildMessage ()) : nullptr,
 		    shared_from_this (), outboundTunnel);
 		if (tunnel->IsEstablished ()) // zero hops
 			TunnelCreated (tunnel);
