@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2020, The PurpleI2P Project
+* Copyright (c) 2013-2021, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -65,7 +65,7 @@ namespace client
 			std::string samAddr; i2p::config::GetOption("sam.address", samAddr);
 			uint16_t    samPort; i2p::config::GetOption("sam.port",    samPort);
 			bool singleThread; i2p::config::GetOption("sam.singlethread", singleThread);
-			LogPrint(eLogInfo, "Clients: starting SAM bridge at ", samAddr, ":", samPort);
+			LogPrint(eLogInfo, "Clients: Starting SAM bridge at ", samAddr, ":", samPort);
 			try
 			{
 				m_SamBridge = new SAMBridge (samAddr, samPort, singleThread);
@@ -83,7 +83,7 @@ namespace client
 		if (bob) {
 			std::string bobAddr; i2p::config::GetOption("bob.address", bobAddr);
 			uint16_t    bobPort; i2p::config::GetOption("bob.port",    bobPort);
-			LogPrint(eLogInfo, "Clients: starting BOB command channel at ", bobAddr, ":", bobPort);
+			LogPrint(eLogInfo, "Clients: Starting BOB command channel at ", bobAddr, ":", bobPort);
 			try
 			{
 				m_BOBCommandChannel = new BOBCommandChannel (bobAddr, bobPort);
@@ -103,7 +103,7 @@ namespace client
 			std::string i2cpAddr; i2p::config::GetOption("i2cp.address", i2cpAddr);
 			uint16_t i2cpPort; i2p::config::GetOption("i2cp.port", i2cpPort);
 			bool singleThread; i2p::config::GetOption("i2cp.singlethread", singleThread);
-			LogPrint(eLogInfo, "Clients: starting I2CP at ", i2cpAddr, ":", i2cpPort);
+			LogPrint(eLogInfo, "Clients: Starting I2CP at ", i2cpAddr, ":", i2cpPort);
 			try
 			{
 				m_I2CPServer = new I2CPServer (i2cpAddr, i2cpPort, singleThread);
@@ -130,7 +130,7 @@ namespace client
 	{
 		if (m_HttpProxy)
 		{
-			LogPrint(eLogInfo, "Clients: stopping HTTP Proxy");
+			LogPrint(eLogInfo, "Clients: Stopping HTTP Proxy");
 			m_HttpProxy->Stop();
 			delete m_HttpProxy;
 			m_HttpProxy = nullptr;
@@ -138,7 +138,7 @@ namespace client
 
 		if (m_SocksProxy)
 		{
-			LogPrint(eLogInfo, "Clients: stopping SOCKS Proxy");
+			LogPrint(eLogInfo, "Clients: Stopping SOCKS Proxy");
 			m_SocksProxy->Stop();
 			delete m_SocksProxy;
 			m_SocksProxy = nullptr;
@@ -146,21 +146,21 @@ namespace client
 
 		for (auto& it: m_ClientTunnels)
 		{
-			LogPrint(eLogInfo, "Clients: stopping I2P client tunnel on port ", it.first);
+			LogPrint(eLogInfo, "Clients: Stopping I2P client tunnel on port ", it.first);
 			it.second->Stop ();
 		}
 		m_ClientTunnels.clear ();
 
 		for (auto& it: m_ServerTunnels)
 		{
-			LogPrint(eLogInfo, "Clients: stopping I2P server tunnel");
+			LogPrint(eLogInfo, "Clients: Stopping I2P server tunnel");
 			it.second->Stop ();
 		}
 		m_ServerTunnels.clear ();
 
 		if (m_SamBridge)
 		{
-			LogPrint(eLogInfo, "Clients: stopping SAM bridge");
+			LogPrint(eLogInfo, "Clients: Stopping SAM bridge");
 			m_SamBridge->Stop ();
 			delete m_SamBridge;
 			m_SamBridge = nullptr;
@@ -168,7 +168,7 @@ namespace client
 
 		if (m_BOBCommandChannel)
 		{
-			LogPrint(eLogInfo, "Clients: stopping BOB command channel");
+			LogPrint(eLogInfo, "Clients: Stopping BOB command channel");
 			m_BOBCommandChannel->Stop ();
 			delete m_BOBCommandChannel;
 			m_BOBCommandChannel = nullptr;
@@ -176,16 +176,16 @@ namespace client
 
 		if (m_I2CPServer)
 		{
-			LogPrint(eLogInfo, "Clients: stopping I2CP");
+			LogPrint(eLogInfo, "Clients: Stopping I2CP");
 			m_I2CPServer->Stop ();
 			delete m_I2CPServer;
 			m_I2CPServer = nullptr;
 		}
 
-		LogPrint(eLogInfo, "Clients: stopping AddressBook");
+		LogPrint(eLogInfo, "Clients: Stopping AddressBook");
 		m_AddressBook.Stop ();
 
-	{
+		{
 			std::lock_guard<std::mutex> lock(m_ForwardsMutex);
 			m_ServerForwards.clear();
 			m_ClientForwards.clear();
@@ -200,6 +200,8 @@ namespace client
 		for (auto& it: m_Destinations)
 			it.second->Stop ();
 		m_Destinations.clear ();
+
+		m_SharedLocalDestination->Release ();
 		m_SharedLocalDestination = nullptr;
 	}
 
@@ -209,14 +211,6 @@ namespace client
 		/*std::string config; i2p::config::GetOption("conf", config);
 		i2p::config::ParseConfig(config);*/
 
-		// handle tunnels
-		// reset isUpdated for each tunnel
-		VisitTunnels ([](I2PService * s)->bool { s->isUpdated = false; return true; });
-		// reload tunnels
-		ReadTunnels();
-		// delete not updated tunnels (not in config anymore)
-		VisitTunnels ([](I2PService * s)->bool { return s->isUpdated; });
-
 		// change shared local destination
 		m_SharedLocalDestination->Release ();
 		CreateNewSharedLocalDestination ();
@@ -225,6 +219,7 @@ namespace client
 		if (m_HttpProxy)
 		{
 			m_HttpProxy->Stop ();
+			delete m_HttpProxy;
 			m_HttpProxy = nullptr;
 		}
 		ReadHttpProxy ();
@@ -233,10 +228,19 @@ namespace client
 		if (m_SocksProxy)
 		{
 			m_SocksProxy->Stop ();
+			delete m_SocksProxy;
 			m_SocksProxy = nullptr;
 		}
 		ReadSocksProxy ();
 
+		// handle tunnels
+		// reset isUpdated for each tunnel
+		VisitTunnels (false);
+		// reload tunnels
+		ReadTunnels();
+		// delete not updated tunnels (not in config anymore)
+		VisitTunnels (true);
+
 		// delete unused destinations
 		std::unique_lock<std::mutex> l(m_DestinationsMutex);
 		for (auto it = m_Destinations.begin (); it != m_Destinations.end ();)
@@ -274,7 +278,7 @@ namespace client
 			s.read ((char *)buf, len);
 			if(!keys.FromBuffer (buf, len))
 			{
-				LogPrint (eLogError, "Clients: failed to load keyfile ", filename);
+				LogPrint (eLogError, "Clients: Failed to load keyfile ", filename);
 				success = false;
 			}
 			else
@@ -283,7 +287,7 @@ namespace client
 		}
 		else
 		{
-			LogPrint (eLogError, "Clients: can't open file ", fullPath, " Creating new one with signature type ", sigType, " crypto type ", cryptoType);
+			LogPrint (eLogError, "Clients: Can't open file ", fullPath, " Creating new one with signature type ", sigType, " crypto type ", cryptoType);
 			keys = i2p::data::PrivateKeys::CreateRandomKeys (sigType, cryptoType);
 			std::ofstream f (fullPath, std::ofstream::binary | std::ofstream::out);
 			size_t len = keys.GetFullLen ();
@@ -402,7 +406,7 @@ namespace client
 
 	void ClientContext::CreateNewSharedLocalDestination ()
 	{
-		std::map<std::string, std::string> params 
+		std::map<std::string, std::string> params
 		{
 			{ I2CP_PARAM_INBOUND_TUNNELS_QUANTITY, "3" },
 			{ I2CP_PARAM_OUTBOUND_TUNNELS_QUANTITY, "3" },
@@ -504,20 +508,15 @@ namespace client
 		int numClientTunnels = 0, numServerTunnels = 0;
 		std::string tunConf; i2p::config::GetOption("tunconf", tunConf);
 		if (tunConf.empty ())
-		{
-			// TODO: cleanup this in 2.8.0
-			tunConf = i2p::fs::DataDirPath ("tunnels.cfg");
-			if (i2p::fs::Exists(tunConf))
-				LogPrint(eLogWarning, "Clients: please rename tunnels.cfg -> tunnels.conf here: ", tunConf);
-			else
-				tunConf = i2p::fs::DataDirPath ("tunnels.conf");
-		}
-		LogPrint(eLogDebug, "Clients: tunnels config file: ", tunConf);
+			tunConf = i2p::fs::DataDirPath ("tunnels.conf");
+
+		LogPrint(eLogDebug, "Clients: Tunnels config file: ", tunConf);
 		ReadTunnels (tunConf, numClientTunnels, numServerTunnels);
 
 		std::string tunDir; i2p::config::GetOption("tunnelsdir", tunDir);
 		if (tunDir.empty ())
 			tunDir = i2p::fs::DataDirPath ("tunnels.d");
+
 		if (i2p::fs::Exists (tunDir))
 		{
 			std::vector<std::string> files;
@@ -526,7 +525,7 @@ namespace client
 				for (auto& it: files)
 				{
 					if (it.substr(it.size() - 5) != ".conf") continue; // skip files which not ends with ".conf"
-					LogPrint(eLogDebug, "Clients: tunnels extra config file: ", it);
+					LogPrint(eLogDebug, "Clients: Tunnels extra config file: ", it);
 					ReadTunnels (it, numClientTunnels, numServerTunnels);
 				}
 			}
@@ -582,7 +581,7 @@ namespace client
 						if (it != destinations.end ())
 							localDestination = it->second;
 						else
-						{	
+						{
 							i2p::data::PrivateKeys k;
 							if(LoadPrivateKeys (k, keys, sigType, cryptoType))
 							{
@@ -597,7 +596,7 @@ namespace client
 										destinations[keys] = localDestination;
 								}
 							}
-						}	
+						}
 					}
 
 					if (type == I2P_TUNNELS_SECTION_TYPE_UDPCLIENT) {
@@ -609,10 +608,18 @@ namespace client
 
 						bool gzip = section.second.get (I2P_CLIENT_TUNNEL_GZIP, true);
 						auto clientTunnel = std::make_shared<I2PUDPClientTunnel>(name, dest, end, localDestination, destinationPort, gzip);
-						if(m_ClientForwards.insert(std::make_pair(end, clientTunnel)).second)
+
+						auto ins = m_ClientForwards.insert(std::make_pair(end, clientTunnel));
+						if (ins.second)
+						{
 							clientTunnel->Start();
+							numClientTunnels++;
+						}
 						else
+						{
+							ins.first->second->isUpdated = true;
 							LogPrint(eLogError, "Clients: I2P Client forward for endpoint ", end, " already exists");
+						}
 
 					} else {
 						boost::asio::ip::tcp::endpoint clientEndpoint;
@@ -645,6 +652,13 @@ namespace client
 							auto tun = std::make_shared<I2PClientTunnel> (name, dest, address, port, localDestination, destinationPort);
 							clientTunnel = tun;
 							clientEndpoint = tun->GetLocalEndpoint ();
+
+							uint32_t keepAlive = section.second.get<uint32_t>(I2P_CLIENT_TUNNEL_KEEP_ALIVE_INTERVAL, 0);
+							if (keepAlive)
+							{
+								tun->SetKeepAliveInterval (keepAlive);
+								LogPrint(eLogInfo, "Clients: I2P Client tunnel keep alive interval set to ", keepAlive);
+							}
 						}
 
 						uint32_t timeout = section.second.get<uint32_t>(I2P_CLIENT_TUNNEL_CONNECT_TIMEOUT, 0);
@@ -666,13 +680,16 @@ namespace client
 							if (ins.first->second->GetLocalDestination () != clientTunnel->GetLocalDestination ())
 							{
 								LogPrint (eLogInfo, "Clients: I2P client tunnel destination updated");
+								ins.first->second->Stop ();
 								ins.first->second->SetLocalDestination (clientTunnel->GetLocalDestination ());
+								ins.first->second->Start ();
 							}
 							ins.first->second->isUpdated = true;
 							LogPrint (eLogInfo, "Clients: I2P client tunnel for endpoint ", clientEndpoint, " already exists");
 						}
 					}
 				}
+
 				else if (type == I2P_TUNNELS_SECTION_TYPE_SERVER
 					|| type == I2P_TUNNELS_SECTION_TYPE_HTTP
 					|| type == I2P_TUNNELS_SECTION_TYPE_IRC
@@ -689,7 +706,7 @@ namespace client
 						accessList=section.second.get (I2P_SERVER_TUNNEL_WHITE_LIST, "");
 					std::string hostOverride = section.second.get (I2P_SERVER_TUNNEL_HOST_OVERRIDE, "");
 					std::string webircpass = section.second.get<std::string> (I2P_SERVER_TUNNEL_WEBIRC_PASSWORD, "");
-					bool gzip = section.second.get (I2P_SERVER_TUNNEL_GZIP, true);
+					bool gzip = section.second.get (I2P_SERVER_TUNNEL_GZIP, false);
 					i2p::data::SigningKeyType sigType = section.second.get (I2P_SERVER_TUNNEL_SIGNATURE_TYPE, i2p::data::SIGNING_KEY_TYPE_EDDSA_SHA512_ED25519);
 					i2p::data::CryptoKeyType cryptoType = section.second.get (I2P_CLIENT_TUNNEL_CRYPTO_TYPE, i2p::data::CRYPTO_KEY_TYPE_ELGAMAL);
 
@@ -703,19 +720,24 @@ namespace client
 					std::shared_ptr<ClientDestination> localDestination = nullptr;
 					auto it = destinations.find (keys);
 					if (it != destinations.end ())
+					{
 						localDestination = it->second;
+						localDestination->SetPublic (true);
+					}
 					else
-					{	
+					{
 						i2p::data::PrivateKeys k;
 						if(!LoadPrivateKeys (k, keys, sigType, cryptoType))
 							continue;
 						localDestination = FindLocalDestination (k.GetPublic ()->GetIdentHash ());
 						if (!localDestination)
-						{	
+						{
 							localDestination = CreateNewLocalDestination (k, true, &options);
 							destinations[keys] = localDestination;
-						}	
-					}	
+						}
+						else
+							localDestination->SetPublic (true);
+					}
 					if (type == I2P_TUNNELS_SECTION_TYPE_UDPSERVER)
 					{
 						// udp server tunnel
@@ -727,26 +749,28 @@ namespace client
 								address = "::1";
 							else
 								address = "127.0.0.1";
-						}	
-						auto localAddress = boost::asio::ip::address::from_string(address);	
+						}
+						auto localAddress = boost::asio::ip::address::from_string(address);
 						auto serverTunnel = std::make_shared<I2PUDPServerTunnel>(name, localDestination, localAddress, endpoint, port, gzip);
 						if(!isUniqueLocal)
 						{
-							LogPrint(eLogInfo, "Clients: disabling loopback address mapping");
+							LogPrint(eLogInfo, "Clients: Disabling loopback address mapping");
 							serverTunnel->SetUniqueLocal(isUniqueLocal);
 						}
 						std::lock_guard<std::mutex> lock(m_ForwardsMutex);
-						if(m_ServerForwards.insert(
-							std::make_pair(
-								std::make_pair(
-									localDestination->GetIdentHash(), port),
-								serverTunnel)).second)
+						auto ins = m_ServerForwards.insert(std::make_pair(
+							std::make_pair(localDestination->GetIdentHash(), port),
+							serverTunnel));
+						if (ins.second)
 						{
 							serverTunnel->Start();
 							LogPrint(eLogInfo, "Clients: I2P Server Forward created for UDP Endpoint ", host, ":", port, " bound on ", address, " for ",localDestination->GetIdentHash().ToBase32());
 						}
 						else
-							LogPrint(eLogError, "Clients: I2P Server Forward for destination/port ", m_AddressBook.ToAddress(localDestination->GetIdentHash()), "/", port, "already exists");
+						{
+							ins.first->second->isUpdated = true;
+							LogPrint(eLogError, "Clients: I2P Server Forward for destination/port ", m_AddressBook.ToAddress(localDestination->GetIdentHash()), "/", port, " already exists");
+						}
 
 						continue;
 					}
@@ -763,7 +787,7 @@ namespace client
 						serverTunnel->SetLocalAddress (address);
 					if(!isUniqueLocal)
 					{
-						LogPrint(eLogInfo, "Clients: disabling loopback address mapping");
+						LogPrint(eLogInfo, "Clients: Disabling loopback address mapping");
 						serverTunnel->SetUniqueLocal(isUniqueLocal);
 					}
 					if (accessList.length () > 0)
@@ -795,7 +819,9 @@ namespace client
 						if (ins.first->second->GetLocalDestination () != serverTunnel->GetLocalDestination ())
 						{
 							LogPrint (eLogInfo, "Clients: I2P server tunnel destination updated");
+							ins.first->second->Stop ();
 							ins.first->second->SetLocalDestination (serverTunnel->GetLocalDestination ());
+							ins.first->second->Start ();
 						}
 						ins.first->second->isUpdated = true;
 						LogPrint (eLogInfo, "Clients: I2P server tunnel for destination/port ", m_AddressBook.ToAddress(localDestination->GetIdentHash ()), "/", inPort, " already exists");
@@ -825,7 +851,7 @@ namespace client
 			std::string httpOutProxyURL;       i2p::config::GetOption("httpproxy.outproxy",      httpOutProxyURL);
 			bool        httpAddresshelper;     i2p::config::GetOption("httpproxy.addresshelper", httpAddresshelper);
 			i2p::data::SigningKeyType sigType; i2p::config::GetOption("httpproxy.signaturetype", sigType);
-			LogPrint(eLogInfo, "Clients: starting HTTP Proxy at ", httpProxyAddr, ":", httpProxyPort);
+			LogPrint(eLogInfo, "Clients: Starting HTTP Proxy at ", httpProxyAddr, ":", httpProxyPort);
 			if (httpProxyKeys.length () > 0)
 			{
 				i2p::data::PrivateKeys keys;
@@ -837,7 +863,7 @@ namespace client
 					if (localDestination) localDestination->Acquire ();
 				}
 				else
-					LogPrint(eLogError, "Clients: failed to load HTTP Proxy key");
+					LogPrint(eLogError, "Clients: Failed to load HTTP Proxy key");
 			}
 			try
 			{
@@ -867,12 +893,12 @@ namespace client
 			std::string socksOutProxyAddr;     i2p::config::GetOption("socksproxy.outproxy",         socksOutProxyAddr);
 			uint16_t    socksOutProxyPort;     i2p::config::GetOption("socksproxy.outproxyport",     socksOutProxyPort);
 			i2p::data::SigningKeyType sigType; i2p::config::GetOption("socksproxy.signaturetype",    sigType);
-			LogPrint(eLogInfo, "Clients: starting SOCKS Proxy at ", socksProxyAddr, ":", socksProxyPort);
+			LogPrint(eLogInfo, "Clients: Starting SOCKS Proxy at ", socksProxyAddr, ":", socksProxyPort);
 			if (httpProxyKeys == socksProxyKeys && m_HttpProxy)
 			{
 				localDestination = m_HttpProxy->GetLocalDestination ();
 				localDestination->Acquire ();
-			}	
+			}
 			else if (socksProxyKeys.length () > 0)
 			{
 				i2p::data::PrivateKeys keys;
@@ -884,7 +910,7 @@ namespace client
 					if (localDestination) localDestination->Acquire ();
 				}
 				else
-					LogPrint(eLogError, "Clients: failed to load SOCKS Proxy key");
+					LogPrint(eLogError, "Clients: Failed to load SOCKS Proxy key");
 			}
 			try
 			{
@@ -920,27 +946,52 @@ namespace client
 		}
 	}
 
-	template<typename Container, typename Visitor>
-	void VisitTunnelsContainer (Container& c, Visitor v)
+	void ClientContext::VisitTunnels (bool clean)
 	{
-		for (auto it = c.begin (); it != c.end ();)
+		for (auto it = m_ClientTunnels.begin (); it != m_ClientTunnels.end ();)
 		{
-			if (!v (it->second.get ()))
-			{
+			if(clean && !it->second->isUpdated) {
 				it->second->Stop ();
-				it = c.erase (it);
-			}
-			else
+				it = m_ClientTunnels.erase(it);
+			} else {
+				it->second->isUpdated = false;
 				it++;
+			}
 		}
-	}
 
-	template<typename Visitor>
-	void ClientContext::VisitTunnels (Visitor v)
-	{
-		VisitTunnelsContainer (m_ClientTunnels, v);
-		VisitTunnelsContainer (m_ServerTunnels, v);
-		// TODO: implement UDP forwards
+		for (auto it = m_ServerTunnels.begin (); it != m_ServerTunnels.end ();)
+		{
+			if(clean && !it->second->isUpdated) {
+				it->second->Stop ();
+				it = m_ServerTunnels.erase(it);
+			} else {
+				it->second->isUpdated = false;
+				it++;
+			}
+		}
+
+		/* // TODO: Write correct UDP tunnels stop
+		for (auto it = m_ClientForwards.begin (); it != m_ClientForwards.end ();)
+		{
+			if(clean && !it->second->isUpdated) {
+				it->second = nullptr;
+				it = m_ClientForwards.erase(it);
+			} else {
+				it->second->isUpdated = false;
+				it++;
+			}
+		}
+
+		for (auto it = m_ServerForwards.begin (); it != m_ServerForwards.end ();)
+		{
+			if(clean && !it->second->isUpdated) {
+				it->second = nullptr;
+				it = m_ServerForwards.erase(it);
+			} else {
+				it->second->isUpdated = false;
+				it++;
+			}
+		} */
 	}
 }
 }
